/*
 * Copyright (C) 2010 Teleal GmbH, Switzerland
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.teleal.cling.test.model;

import org.teleal.cling.model.ModelUtil;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;


public class UtilTest {

    @Test
    public void csvToString() {

        Object[] plainStrings = new Object[] {"foo", "bar", "baz"};
        assertEquals(ModelUtil.toCommaSeparatedList(plainStrings), "foo,bar,baz");

        Object[] commaStrings = new Object[] {"foo,", "bar", "b,az"};
        assertEquals(ModelUtil.toCommaSeparatedList(commaStrings), "foo\\,,bar,b\\,az");

        Object[] backslashStrings = new Object[] {"f\\oo", "b,ar", "b\\az"};
        assertEquals(ModelUtil.toCommaSeparatedList(backslashStrings), "f\\\\oo,b\\,ar,b\\\\az");
    }

    @Test
    public void stringToCsv() {

        Object[] plainStrings = new Object[] {"foo", "bar", "baz"};
        assertEquals(ModelUtil.fromCommaSeparatedList("foo,bar,baz"), plainStrings);

        Object[] commaStrings = new Object[] {"foo,", "bar", "b,az"};
        assertEquals(ModelUtil.fromCommaSeparatedList("foo\\,,bar,b\\,az"), commaStrings);

        Object[] backslashStrings = new Object[] {"f\\oo", "b,ar", "b\\az"};
        assertEquals(ModelUtil.fromCommaSeparatedList("f\\\\oo,b\\,ar,b\\\\az"), backslashStrings);
    }

}
