/*
 * Copyright (C) 2010 Teleal GmbH, Switzerland
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.teleal.cling.support.avtransport.callback;

import org.teleal.cling.controlpoint.ActionCallback;
import org.teleal.cling.model.action.ActionArgumentValue;
import org.teleal.cling.model.action.ActionException;
import org.teleal.cling.model.action.ActionInvocation;
import org.teleal.cling.model.meta.DeviceService;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.types.UnsignedIntegerFourBytes;
import org.teleal.cling.support.avtransport.dto.MediaInfo;

import java.util.logging.Logger;

/**
 *
 * @author Christian Bauer
 */
public abstract class GetMediaInfoActionCallback extends ActionCallback {

    private static Logger log = Logger.getLogger(GetMediaInfoActionCallback.class.getName());

    public GetMediaInfoActionCallback(DeviceService service) {
        this(new UnsignedIntegerFourBytes(0), service);
    }

    public GetMediaInfoActionCallback(UnsignedIntegerFourBytes instanceId, DeviceService service) {
        super(new ActionInvocation(service.getService().getAction("GetMediaInfo")));

        try {
            getActionInvocation().getInput().addValue(instanceId);
        } catch (ActionException ex) {
            throw new RuntimeException(ex);
        }
    }

    public void success(ActionInvocation actionInvocation) {
        ActionArgumentValue[] output = actionInvocation.getOutput().getValues();
        MediaInfo mediaInfo = new MediaInfo(output);
        received(actionInvocation, mediaInfo);
    }

    public void failure(ActionInvocation actionInvocation, UpnpResponse operation) {
        failure(createDefaultFailureMessage(actionInvocation, operation));
    }

    public abstract void received(ActionInvocation actionInvocation, MediaInfo mediaInfo);
    public abstract void failure(String failureMessage);

}