/*
 * Copyright (C) 2010 Teleal GmbH, Switzerland
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.teleal.cling.support.connectionmanager;

import java.util.logging.Logger;
import org.teleal.cling.controlpoint.ActionCallback;
import org.teleal.cling.model.action.ActionArgumentValue;
import org.teleal.cling.model.action.ActionException;
import org.teleal.cling.model.action.ActionInvocation;
import org.teleal.cling.model.meta.DeviceService;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.types.ErrorCode;
import org.teleal.cling.support.connectionmanager.protocolinfo.ProtocolInfo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 *
 * @author Christian Bauer
 */
public abstract class GetProtocolInfoActionCallback extends ActionCallback {

    private static Logger log = Logger.getLogger(GetProtocolInfoActionCallback.class.getName());

    public GetProtocolInfoActionCallback(DeviceService service) {
        super(new ActionInvocation(service.getService().getAction("GetProtocolInfo")));
    }

    public void success(ActionInvocation actionInvocation) {
        ActionArgumentValue[] output = actionInvocation.getOutput().getValues();

        String protocolInfosString = output[1].toString();

        if (protocolInfosString.length() > 0) {
            try {

                List<ProtocolInfo> protocolInfos = new ArrayList();
                String[] split = protocolInfosString.split(",");
                for (String s : split) {
                    protocolInfos.add(ProtocolInfo.fromString(s));
                }

                received(actionInvocation, protocolInfos);

            } catch (Exception ex) {
                actionInvocation.setFailure(
                        new ActionException(ErrorCode.ACTION_FAILED, "Can't parse ProtocolInfo response: " + ex, ex)
                );
                failure(actionInvocation, null);
            }
        } else {
            received(actionInvocation, Collections.EMPTY_LIST);
        }
    }

    public void failure(ActionInvocation actionInvocation, UpnpResponse operation) {
        failure(createDefaultFailureMessage(actionInvocation, operation));
    }

    public abstract void received(ActionInvocation actionInvocation, List<ProtocolInfo> protocolInfos);
    public abstract void failure(String failureMessage);

}